@@ -122,7 +122,8 @@ void XRandRConfig::applyKScreenConfig(KScreen::Config *config)
         XRandROutput *currentOutput = m_outputs.value(output->id());
         currentOutput->update(currentOutput->isPrimary() ? XRandROutput::SetPrimary : XRandROutput::UnsetPrimary);
 
-        if (output->isPrimary()) {
+        //Only set the output as primary if it is enabled.
+        if (output->isPrimary() && output->isEnabled()) {
             primaryOutput = currentOutput->id();
         }
 
@@ -161,7 +162,10 @@ void XRandRConfig::applyKScreenConfig(KScreen::Config *config)
             }
         }
 
-        QSize size = output->mode(currentOutput->currentModeId())->size();
+        XRandRMode* currentMode = currentOutput->currentMode();
+        Q_ASSERT_X(currentMode, "applyKScreenConfig", "currentOutput has returned a null XRandRMode*");
+
+        QSize size = currentMode->size();
 
         int x, y;
 
@@ -231,6 +235,9 @@ void XRandRConfig::applyKScreenConfig(KScreen::Config *config)
 
     //If there is nothing to do, not even bother
     if (toDisable.isEmpty() && toEnable.isEmpty() && toChange.isEmpty()) {
+        if (newSize != m_screen->currentSize()) {
+            setScreenSize(newSize);
+        }
         return;
     }
 
